@@ -119,6 +119,7 @@ pub(crate) fn eval_expr(
         Expr::FnCall(identifier, expressions) => {
             eval_fn_call_expr(context, identifier, expressions, unit)
         }
+        Expr::Piecewise(pieces) => eval_piecewise(context, pieces, unit),
     }
 }
 
@@ -142,10 +143,9 @@ fn eval_binary_expr(
     let left = eval_expr(context, left_expr, "")?;
     let mut right = eval_expr(context, right_expr, "")?;
     if let Expr::Unary(TokenKind::Percent, _) = right_expr {
+        right = right.mul(context, left.clone());
         if let TokenKind::Star = op {
-            right = right.mul(context, KalkNum::from(0.01));
-        } else {
-            right = right.mul(context, left.clone());
+            return Ok(right);
         }
     }
 
@@ -309,25 +309,39 @@ pub(crate) fn eval_fn_call_expr(
 
     // Special functions
     match identifier.full_name.as_ref() {
-        "sum" | "Σ" => {
+        "sum" | "prod" => {
             // Make sure exactly 3 arguments were supplied.
             if expressions.len() != 3 {
                 return Err(CalcError::IncorrectAmountOfArguments(
                     3,
-                    "sum".into(),
+                    "sum/prod".into(),
                     expressions.len(),
                 ));
             }
 
             let start = eval_expr(context, &expressions[0], "")?.to_f64() as i128;
             let end = eval_expr(context, &expressions[1], "")?.to_f64() as i128;
-            let mut sum = KalkNum::default();
+            let sum_else_prod = match identifier.full_name.as_ref() {
+                "sum" => true,
+                "prod" => false,
+                _ => unreachable!(),
+            };
+            let mut sum = if sum_else_prod {
+                KalkNum::default()
+            } else {
+                KalkNum::from(1f64)
+            };
 
             for n in start..=end {
                 context.sum_n_value = Some(n);
                 let eval = eval_expr(context, &expressions[2], "")?;
-                sum.value += eval.value;
-                sum.imaginary_value += eval.imaginary_value;
+                if sum_else_prod {
+                    sum.value += eval.value;
+                    sum.imaginary_value += eval.imaginary_value;
+                } else {
+                    sum.value *= eval.value;
+                    sum.imaginary_value *= eval.imaginary_value;
+                }
             }
 
             context.sum_n_value = None;
@@ -335,7 +349,7 @@ pub(crate) fn eval_fn_call_expr(
 
             return Ok(sum);
         }
-        "integrate" | "integral" | "∫" => {
+        "integrate" => {
             // Make sure either 3 or 4 arguments were supplied.
             if expressions.len() < 3 || expressions.len() > 4 {}
 
@@ -381,22 +395,73 @@ pub(crate) fn eval_fn_call_expr(
             }
 
             // Initialise the arguments as their own variables.
+            let mut new_argument_values = Vec::new();
             for (i, argument) in arguments.iter().enumerate() {
-                eval_stmt(
-                    context,
-                    &Stmt::VarDecl(
-                        Identifier::from_full_name(argument),
-                        Box::new(expressions[i].clone()),
-                    ),
-                )?;
+                let argument_identifier = if argument.contains("-") {
+                    let identifier_parts: Vec<&str> = argument.split('-').collect();
+                    Identifier::parameter_from_name(identifier_parts[1], identifier_parts[0])
+                } else {
+                    Identifier::from_full_name(argument)
+                };
+                let var_decl = Stmt::VarDecl(
+                    argument_identifier,
+                    Box::new(crate::ast::build_literal_ast(&eval_expr(
+                        context,
+                        &expressions[i],
+                        "",
+                    )?)),
+                );
+
+                // Don't set these values just yet, since
+                // to avoid affecting the value of arguments
+                // during recursion.
+                new_argument_values.push((argument, var_decl));
             }
 
-            eval_expr(context, &fn_body, unit)
+            let mut old_argument_values = Vec::new();
+            for (name, value) in new_argument_values {
+                // Save the original argument values,
+                // so that they can be reverted to after
+                // the function call is evaluated.
+                // This is necessary since recursive
+                // function calls have the same argument names.
+                old_argument_values.push(context.symbol_table.get_and_remove_var(name));
+
+                // Now set the new variable value
+                eval_stmt(context, &value)?;
+            }
+
+            let fn_value = eval_expr(context, &fn_body, unit);
+
+            // Revert to original argument values
+            for old_argument_value in old_argument_values {
+                if let Some(old_argument_value) = old_argument_value {
+                    context.symbol_table.insert(old_argument_value);
+                }
+            }
+
+            fn_value
         }
         _ => Err(CalcError::UndefinedFn(identifier.full_name.clone())),
     }
 }
 
+fn eval_piecewise(
+    context: &mut Context,
+    pieces: &Vec<crate::ast::ConditionalPiece>,
+    unit: &str,
+) -> Result<KalkNum, CalcError> {
+    for piece in pieces {
+        if let Some(condition_is_true) = eval_expr(context, &piece.condition, unit)?.boolean_value {
+            if condition_is_true {
+                return Ok(eval_expr(context, &piece.expr, unit)?);
+            }
+        }
+    }
+
+    Err(CalcError::PiecewiseConditionsAreFalse)
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -490,37 +555,37 @@ mod tests {
         let result = interpret(equals).unwrap().unwrap();
         assert_eq!(
             (result.to_f64(), result.boolean_value.unwrap()),
-            (3f64, false)
+            (0f64, false)
         );
 
         let result = interpret(not_equals).unwrap().unwrap();
         assert_eq!(
             (result.to_f64(), result.boolean_value.unwrap()),
-            (3f64, true)
+            (0f64, true)
         );
 
         let result = interpret(greater_than).unwrap().unwrap();
         assert_eq!(
             (result.to_f64(), result.boolean_value.unwrap()),
-            (3f64, false)
+            (0f64, false)
         );
 
         let result = interpret(less_than).unwrap().unwrap();
         assert_eq!(
             (result.to_f64(), result.boolean_value.unwrap()),
-            (3f64, true)
+            (0f64, true)
         );
 
         let result = interpret(greater_or_equals).unwrap().unwrap();
         assert_eq!(
             (result.to_f64(), result.boolean_value.unwrap()),
-            (3f64, false)
+            (0f64, false)
         );
 
         let result = interpret(less_or_equals).unwrap().unwrap();
         assert_eq!(
             (result.to_f64(), result.boolean_value.unwrap()),
-            (3f64, true)
+            (0f64, true)
         );
     }
 
