@@ -17,6 +17,8 @@ use lazy_static::lazy_static;
 use std::collections::HashMap;
 use wasm_bindgen::prelude::*;
 
+const ACCEPTABLE_COMPARISON_MARGIN: f64 = 0.00000001;
+
 lazy_static! {
     static ref CONSTANTS: HashMap<&'static str, &'static str> = {
         let mut m = HashMap::new();
@@ -186,6 +188,10 @@ impl KalkNum {
     }
 
     pub fn to_string_pretty(&self) -> String {
+        if let Some(boolean_value) = self.boolean_value {
+            return boolean_value.to_string();
+        }
+
         let real_f64 = self.to_f64();
         let imaginary_f64 = self.imaginary_to_f64();
         if real_f64.is_nan() || imaginary_f64.is_nan() {
@@ -201,7 +207,7 @@ impl KalkNum {
         let result_str = if (-6..8).contains(&sci_notation_real.exponent) || self.value == 0f64 {
             self.to_string_real()
         } else if sci_notation_real.exponent <= -14 {
-            adjusted_num.value = KalkNum::from(1f64).value;
+            adjusted_num.value = KalkNum::from(0f64).value;
             String::from("0")
         } else {
             sci_notation_real.to_string().trim().to_string()
@@ -314,6 +320,59 @@ impl KalkNum {
         KalkNum::new(self.value % right.value, &right.unit)
     }
 
+    pub(crate) fn eq(self, context: &mut crate::interpreter::Context, rhs: KalkNum) -> KalkNum {
+        let right = calculate_unit(context, &self, rhs.clone()).unwrap_or(rhs);
+        self.eq_without_unit(&right)
+    }
+
+    pub(crate) fn not_eq(self, context: &mut crate::interpreter::Context, rhs: KalkNum) -> KalkNum {
+        let right = calculate_unit(context, &self, rhs.clone()).unwrap_or(rhs);
+        self.not_eq_without_unit(&right)
+    }
+
+    pub(crate) fn greater_than(
+        self,
+        context: &mut crate::interpreter::Context,
+        rhs: KalkNum,
+    ) -> KalkNum {
+        let right = calculate_unit(context, &self, rhs.clone()).unwrap_or(rhs);
+        self.greater_than_without_unit(&right)
+    }
+
+    pub(crate) fn less_than(
+        self,
+        context: &mut crate::interpreter::Context,
+        rhs: KalkNum,
+    ) -> KalkNum {
+        let right = calculate_unit(context, &self, rhs.clone()).unwrap_or(rhs);
+        self.less_than_without_unit(&right)
+    }
+
+    pub(crate) fn greater_or_equals(
+        self,
+        context: &mut crate::interpreter::Context,
+        rhs: KalkNum,
+    ) -> KalkNum {
+        let right = calculate_unit(context, &self, rhs.clone()).unwrap_or(rhs.clone());
+        let greater = self
+            .greater_than_without_unit(&right)
+            .boolean_value
+            .unwrap();
+        let equal = self.eq_without_unit(&right).boolean_value.unwrap();
+        KalkNum::from_bool(greater || equal)
+    }
+
+    pub(crate) fn less_or_equals(
+        self,
+        context: &mut crate::interpreter::Context,
+        rhs: KalkNum,
+    ) -> KalkNum {
+        let right = calculate_unit(context, &self, rhs.clone()).unwrap_or(rhs.clone());
+        let less = self.less_than_without_unit(&right).boolean_value.unwrap();
+        let equal = self.eq_without_unit(&right).boolean_value.unwrap();
+        KalkNum::from_bool(less || equal)
+    }
+
     pub(crate) fn add_without_unit(self, rhs: KalkNum) -> KalkNum {
         KalkNum::new_with_imaginary(
             self.value + rhs.value,
@@ -358,6 +417,42 @@ impl KalkNum {
         }
     }
 
+    pub(crate) fn eq_without_unit(&self, rhs: &KalkNum) -> KalkNum {
+        if self.has_imaginary() || rhs.has_imaginary() {
+            return KalkNum::from(f64::NAN);
+        }
+
+        KalkNum::from_bool(
+            (self.value.clone() - rhs.value.clone()).abs() < ACCEPTABLE_COMPARISON_MARGIN,
+        )
+    }
+
+    pub(crate) fn not_eq_without_unit(&self, rhs: &KalkNum) -> KalkNum {
+        if self.has_imaginary() || rhs.has_imaginary() {
+            return KalkNum::from(f64::NAN);
+        }
+
+        KalkNum::from_bool(
+            (self.value.clone() - rhs.value.clone()).abs() > ACCEPTABLE_COMPARISON_MARGIN,
+        )
+    }
+
+    pub(crate) fn greater_than_without_unit(&self, rhs: &KalkNum) -> KalkNum {
+        if self.has_imaginary() || rhs.has_imaginary() {
+            return KalkNum::from(f64::NAN);
+        }
+
+        KalkNum::from_bool(self.value.clone() - rhs.value.clone() > ACCEPTABLE_COMPARISON_MARGIN)
+    }
+
+    pub(crate) fn less_than_without_unit(&self, rhs: &KalkNum) -> KalkNum {
+        if self.has_imaginary() || rhs.has_imaginary() {
+            return KalkNum::from(f64::NAN);
+        }
+
+        KalkNum::from_bool(self.value.clone() - rhs.value.clone() < -ACCEPTABLE_COMPARISON_MARGIN)
+    }
+
     pub fn get_conjugate(&self) -> KalkNum {
         KalkNum::new_with_imaginary(
             self.value.clone(),
@@ -790,6 +885,7 @@ mod tests {
             (10e-17, 1.0, "i"),
             (1.0, 10e-17, "1"),
             (10e-15, 0.0, "0"),
+            (3.00000000004, 0.0, "3"),
         ];
         for (real, imaginary, output) in in_out {
             let result = KalkNum::new_with_imaginary(
