@@ -97,18 +97,22 @@ impl Default for Context {
 /// Error that occured during parsing or evaluation.
 #[derive(Debug, Clone, PartialEq)]
 pub enum CalcError {
+    Expected(String),
     ExpectedDx,
+    ExpectedIf,
     IncorrectAmountOfArguments(usize, String, usize),
     InvalidNumberLiteral(String),
     InvalidOperator,
     InvalidUnit,
     TimedOut,
     VariableReferencesItself,
+    PiecewiseConditionsAreFalse,
     UnexpectedToken(TokenKind, TokenKind),
     UndefinedFn(String),
     UndefinedVar(String),
     UnableToInvert(String),
     UnableToSolveEquation,
+    UnableToOverrideConstant(String),
     UnableToParseExpression,
     Unknown,
 }
@@ -116,7 +120,9 @@ pub enum CalcError {
 impl ToString for CalcError {
     fn to_string(&self) -> String {
         match self {
+            CalcError::Expected(description) => format!("Expected: {}", description),
             CalcError::ExpectedDx => format!("Expected eg. dx, to specify for which variable the operation is being done to. Example with integration: ∫(0, 1, x dx) or ∫(0, 1, x, dx). You may need to put parenthesis around the expression before dx/dy/du/etc."),
+            CalcError::ExpectedIf => format!("Expected 'if', with a condition after it."),
             CalcError::IncorrectAmountOfArguments(expected, func, got) => format!(
                 "Expected {} arguments for function {}, but got {}.",
                 expected, func, got
@@ -126,6 +132,7 @@ impl ToString for CalcError {
             CalcError::InvalidUnit => format!("Invalid unit."),
             CalcError::TimedOut => format!("Operation took too long."),
             CalcError::VariableReferencesItself => format!("Variable references itself."),
+            CalcError::PiecewiseConditionsAreFalse => format!("All the conditions in the piecewise are false."),
             CalcError::UnexpectedToken(got, expected) => {
                 format!("Unexpected token: '{:?}', expected '{:?}'.", got, expected)
             }
@@ -134,6 +141,7 @@ impl ToString for CalcError {
             CalcError::UndefinedVar(name) => format!("Undefined variable: '{}'.", name),
             CalcError::UnableToParseExpression => format!("Unable to parse expression."),
             CalcError::UnableToSolveEquation => format!("Unable to solve equation."),
+            CalcError::UnableToOverrideConstant(name) => format!("Unable to override constant: '{}'.", name),
             CalcError::Unknown => format!("Unknown error."),
         }
     }
@@ -151,7 +159,8 @@ pub fn eval(
     // if the equal sign is for one of those instead.
     // It also should not contain an iverson bracket, since equal signs in there
     // mean something else. This is not super reliable, and should probably be improved in the future.
-    context.contains_equation_equal_sign = input.contains("=") && !input.contains("[");
+    context.contains_equation_equal_sign =
+        input.contains("=") && !input.contains("[") && !input.contains("{");
     let statements = parse(context, input)?;
 
     let mut interpreter = interpreter::Context::new(
@@ -234,7 +243,13 @@ fn parse_identifier_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
                 context.current_function_parameters = Some(parameter_identifiers.clone());
                 context.contains_equation_equal_sign = false;
                 context.equation_variable = None;
-                let expr = parse_expr(context)?;
+
+                // Piecewise
+                let expr = if match_token(context, TokenKind::OpenBrace) {
+                    parse_piecewise(context)?
+                } else {
+                    parse_expr(context)?
+                };
                 let fn_decl = Stmt::FnDecl(identifier, parameter_identifiers, Box::new(expr));
                 context.current_function = None;
                 context.current_function_parameters = None;
@@ -254,6 +269,46 @@ fn parse_identifier_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
     Ok(Stmt::Expr(Box::new(parse_expr(context)?)))
 }
 
+fn parse_piecewise(context: &mut Context) -> Result<Expr, CalcError> {
+    advance(context);
+
+    let mut pieces = Vec::new();
+    let mut reached_otherwise = false;
+    // Parse `expr if condition`
+    // do-while loop
+    while {
+        let left_expr = parse_expr(context)?;
+        if match_token(context, TokenKind::IfKeyword) {
+            advance(context);
+            pieces.push(crate::ast::ConditionalPiece {
+                expr: left_expr,
+                condition: parse_expr(context)?,
+            });
+        } else if match_token(context, TokenKind::OtherwiseKeyword) {
+            advance(context);
+            // Yeah, a bit hacky, but there's no `true` keyword...
+            let true_expr = Expr::Binary(
+                Box::new(Expr::Literal(1f64)),
+                TokenKind::Equals,
+                Box::new(Expr::Literal(1f64)),
+            );
+            pieces.push(crate::ast::ConditionalPiece {
+                expr: left_expr,
+                condition: true_expr,
+            });
+
+            reached_otherwise = true;
+        } else {
+            return Err(CalcError::ExpectedIf);
+        }
+
+        advance(context);
+        previous(context).kind == TokenKind::Semicolon && !reached_otherwise
+    } {}
+
+    Ok(Expr::Piecewise(pieces))
+}
+
 fn parse_var_decl_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
     let identifier = advance(context).clone();
     advance(context); // Equal sign
@@ -264,6 +319,10 @@ fn parse_var_decl_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
         return Err(CalcError::VariableReferencesItself);
     }
 
+    if prelude::is_constant(&identifier.value) {
+        return Err(CalcError::UnableToOverrideConstant(identifier.value.into()));
+    }
+
     Ok(Stmt::VarDecl(
         Identifier::from_full_name(&identifier.value),
         Box::new(expr),
@@ -516,6 +575,13 @@ fn parse_group_fn(context: &mut Context) -> Result<Expr, CalcError> {
     };
 
     let expr = parse_expr(context)?;
+
+    if peek(context).kind == TokenKind::EOF {
+        return Err(CalcError::Expected(String::from(
+            "Closing group symbol, eg. ⌋",
+        )));
+    }
+
     advance(context);
 
     Ok(Expr::FnCall(Identifier::from_full_name(name), vec![expr]))
@@ -523,27 +589,25 @@ fn parse_group_fn(context: &mut Context) -> Result<Expr, CalcError> {
 
 fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
     let identifier = Identifier::from_full_name(&advance(context).value);
+    let exists_as_fn = context.symbol_table.contains_fn(&identifier.pure_name)
+        || context.current_function.as_ref() == Some(&identifier.pure_name);
 
     // Eg. sqrt64
-    if match_token(context, TokenKind::Literal)
-        || peek(context).value == "π"
-        || peek(context).value == "τ"
-        || peek(context).value == "ϕ"
+    if exists_as_fn
+        && (match_token(context, TokenKind::Literal) || match_token(context, TokenKind::Identifier))
     {
         // If there is a function with this name, parse it as a function, with the next token as the argument.
-        if context.symbol_table.contains_fn(&identifier.pure_name) {
-            let parameter = if identifier.full_name == "√" {
-                parse_exponent(context)?
-            } else {
-                parse_factor(context)?
-            };
-            return Ok(Expr::FnCall(identifier, vec![parameter]));
-        }
+        let parameter = if identifier.full_name == "√" {
+            parse_exponent(context)?
+        } else {
+            parse_factor(context)?
+        };
+        return Ok(Expr::FnCall(identifier, vec![parameter]));
     }
 
     let parse_as_var_instead = match_token(context, TokenKind::OpenParenthesis)
         && !context.parsing_identifier_stmt
-        && !context.symbol_table.contains_fn(&identifier.pure_name);
+        && !exists_as_fn;
 
     // Eg. sqrt(64)
     // If the function doesn't exist, parse it as a variable and multiplication instead.
@@ -641,7 +705,11 @@ fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
             }
         }
 
-        split_into_variables(context, &identifier)
+        if identifier.pure_name.len() > 1 {
+            split_into_variables(context, &identifier)
+        } else {
+            Err(CalcError::UndefinedVar(identifier.full_name))
+        }
     }
 }
 
@@ -720,7 +788,11 @@ fn build_var(context: &Context, name: &str) -> Expr {
 }
 
 fn peek(context: &Context) -> &Token {
-    &context.tokens[context.pos]
+    if context.pos >= context.tokens.len() {
+        &context.tokens.last().unwrap() // EOF
+    } else {
+        &context.tokens[context.pos]
+    }
 }
 
 fn peek_next(context: &Context) -> &Token {
